from rocolib.api.components.Component import Component
from rocolib.api.Function import Function

c = Component()

#Parameters for the sheath that would cover the wheels and the ESP32 Stack
#Parameters that will change the dimension of the car
c.addParameter("length", 65, paramType="length", minValue=65) #tested minValue
c.addParameter("height", 36, paramType="length", minValue=36) #tested minValue
c.addParameter("width", 60, paramType="length", minValue=60) #tested minValue

#Holder for the ESP32 FeatherBoard and its Featherwings
#If you have another "brain" or microcontroller or board change it here and adjust the parameters above to fit that "brain"
c.addSubcomponent("holder", "SubESP32Stack")
c.addConstraint(("holder", "length"), "width")

#This car will use the customized ESP32 stack in dimensions.py and the FS90R servo
#Change here if you have another board
c.addParameter("driveservo", "fs90r", paramType="dimension")
c.addParameter("brains", "esp32stack", paramType="dimension")

#Four wheels using Wheel.yaml
c.addSubcomponent("fright", "Wheel", invert=True, inherit="angle")
c.addSubcomponent("bleft", "Wheel", invert=True, inherit="angle")

#Setting the parameters for the Wheels
for servo in ("fright", "bleft"):
    c.addConstraint((servo, "servo"), "driveservo")
    c.addConstraint((servo, "length"), ("length", "brains"), '((x[0] - getDim(x[1], "width")))')
    c.addConstConstraint((servo, "center"), False)
    c.addConstraint((servo, "radius"), "brains", '(getDim(x, "height") / 1.35)')

#Sheath to cover the wheels and the ESP32 Stack
c.addSubcomponent("sheath0", "SimpleRectBeam")
c.addConstraint(("sheath0", "length"), "length")
c.addConstraint(("sheath0", "depth"), "height")
c.addConstraint(("sheath0", "width"), "width")

#Support for the middle of the front and back wheels so that they won't collapse or wiggle
c.addSubcomponent("between0", "SimpleRectBeam")
c.addConstraint(("between0", "length"), ("width", "driveservo"), "x[0]- 2 * getDim(x[1], 'motorheight')")
c.addConstraint(("between0", "depth"), "driveservo", "getDim(x, 'motorwidth')")
c.addConstraint(("between0", "width"), "driveservo", "getDim(x, 'motorwidth')")

#Front between support
c.addConnection(("fright", "topedge2"), ("between0", "topedge1"), angle=90)
c.addConnection(("between0", "botedge1"), ("bleft", "botedge2"), angle=90, tabWidth=10)

#Flipping he front right wheels to match the orientation of the
#bottom left
c.addConstConstraint(("fright", "flip"), True)

#Connecting the wheels to attach to the "holder" of the ESP32 stack
for i in range(2):
    c.addSubcomponent("split%d" %i, "SplitEdge")
    c.addConstraint(("split%d" %i, "toplength"), "brains", '(getDim(x, "height"),)')

c.addConstraint(("split0", "botlength"), ("brains", "driveservo"),
                '(getDim(x[1], "motorwidth"), getDim(x[0], "height") - getDim(x[1], "motorwidth"))')

c.addConstraint(("split1", "botlength"), ("brains", "driveservo"),
                '(getDim(x[0], "height") - getDim(x[1], "motorwidth"), getDim(x[1], "motorwidth"))')

c.addConnection(("holder", "botedge1"), ("split0", "topedge0"))
c.addConnection(("split0", "botedge0"), ("fright", "botedge0"), angle=-90)
c.addConnection(("holder", "topedge1"), ("split1", "topedge0"))
c.addConnection(("split1", "botedge1"), ("bleft", "topedge0"), angle=-90)

#Connecting the sheath to the ESP32 stack and wheels
c.addSubcomponent("sheathsplit0", "SplitEdge")
c.addConstraint(("sheathsplit0", "toplength"), "width", "(x,)")
c.addConstraint(("sheathsplit0", "botlength"), ("driveservo", "width"), "(x[1] - getDim(x[0],'motorheight'), \
              getDim(x[0],'motorheight'))")

c.addConnection(("bleft", "botedge1"), ("sheathsplit0", "botedge1"),angle=180)
c.addConnection(("sheathsplit0", "topedge0"), ("sheath0", "topedge2"))

#Creating the holes for the servo wires
for i in range(2):
    c.addSubcomponent("sidehole%d" %i, "Cutout")
    c.addConstraint(("sidehole%d" %i, "dx"), "height", "x * 0.39")

c.addConstraint(("sidehole0", "dy"), "height", "x * 0.7")
c.addConstraint(("sidehole1", "dy"), "height", "x * 0.5")

c.addConnection(("sheath0", "face1"),
                   ("sidehole0", "decoration"),
                   mode="hole", rotate=True, offset=Function(params=("height", "brains"), fnstring="(x[0] * -0.10, getDim(x[1], 'width') - 3)"))
c.addConnection(("sheath0", "face3"),
                   ("sidehole1", "decoration"),
                   mode="hole", rotate=True, offset=Function(params=("height", "brains"), fnstring="(x[0] * -0.04, getDim(x[1], 'width') - 3)"))

for i in range(4):
    c.addSubcomponent("accessoryHole%d" % i, "Cutout")
    c.addConstraint(("accessoryHole%d" % i, "dx"), "height", "x * 0.03")
    c.addConstraint(("accessoryHole%d" % i, "dy"), "height", "x * 0.5")

c.addConnection(("sheath0", "face1"),
                   ("accessoryHole0", "decoration"),
                   mode="hole", rotate=True, offset=Function(params=("height", "length"), fnstring="(x[0] * -0.10, -x[1] * 0.45)"))
c.addConnection(("sheath0", "face3"),
                   ("accessoryHole1", "decoration"),
                   mode="hole", rotate=True, offset=Function(params=("height", "length"), fnstring="(x[0] * -0.04, -x[1] * 0.45)"))

c.addConnection(("sheath0", "face1"),
                   ("accessoryHole2", "decoration"),
                   mode="hole", rotate=True, offset=Function(params=("height", "length"), fnstring="(x[0] * -0.10, x[1] * 0.05)"))
c.addConnection(("sheath0", "face3"),
                   ("accessoryHole3", "decoration"),
                   mode="hole", rotate=True, offset=Function(params=("height", "length"), fnstring="(x[0] * -0.04, x[1] * 0.05)"))

c.inheritAllInterfaces("sheathsplit0")
c.inheritAllInterfaces("sheath0")

c.toLibrary("NewBrainsTwoWheels")